#!perl -w

package DE_EPAGES::Core::t::Git;

use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Git;
use DE_EPAGES::Core::API::File qw (
    ExistsDirectory
    DeleteDirectory
    RunOnDirectory
);
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::More tests => 19;
use Test::Exception;

use strict;
use Git::Repository;

my $TmpRepo = '/tmp/testGit';

sub test {
    my $self = shift;

    my $LocalRepo = $ENV{EPAGES_CARTRIDGES};
    $LocalRepo = $TmpRepo if (ExistsDirectory($TmpRepo));

    RunOnDirectory(
        $LocalRepo,
        sub {
            $self->test_Add_Success();
            $self->test_Remove_Success();
            $self->test_Commit_Die();
            $self->test_Commit_Success();
            $self->test_Checkout_Success();
            $self->test_CreateBranch_Success();
            $self->test_ExistsRemoteBranch_True();
            $self->test_ExistsRemoteBranch_False();
            $self->test_ExistsLocalBranch_True();
            $self->test_ExistsLocalBranch_False();
            $self->test_Merge_Success();
            $self->test_Push_Success();
            done_testing();
        }
    );

}

sub init {
    my $self = shift;

    unless(ExistsDirectory("$ENV{EPAGES_CARTRIDGES}/.git")) {
        Git::Repository->run(init => $TmpRepo);
    }
}

sub cleanup {
    my $self = shift;

    if(ExistsDirectory($TmpRepo)) {
        DeleteDirectory($TmpRepo, {'recursive' => 1});
    }
}

sub test_Add_Success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyFile = "src/dummy.txt";

    my $Git = $self->_createGit($self->_createGitRepository());

    lives_ok( sub { $Git->Add() }, "$SubTestName - Successful without parameters.");
    lives_ok( sub { $Git->Add($DummyFile) }, "$SubTestName - Successful with file." );
}

sub test_Remove_Success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyFile = "src/dummy.txt";

    my $Git = $self->_createGit($self->_createGitRepository());

    lives_ok( sub { $Git->Remove() }, "$SubTestName - Successful without parameters.");
    lives_ok( sub { $Git->Remove($DummyFile) }, "$SubTestName - Successful with file.");
}

sub test_Commit_Die {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Git = $self->_createGit($self->_createGitRepository());

    dies_ok( sub { $Git->Commit() }, "$SubTestName - Dies without message.");
}

sub test_Commit_Success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyMessage = "My fantastic dummy commit message.";

    my $Git = $self->_createGit($self->_createGitRepositoryForClone());

    lives_ok( sub { $Git->Commit($DummyMessage) }, "$SubTestName - Successful with message.");
}

sub test_Checkout_Success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyBranch = "issue/dummyBranch";

    my $Git = $self->_createGit($self->_createGitRepository());

    lives_ok( sub { $Git->Checkout() }, "$SubTestName - Successful without parameters.");
    lives_ok( sub { $Git->Checkout($DummyBranch) }, "$SubTestName - Successful with branch.");
}

sub test_CreateBranch_Success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyBranch = "issue/dummyBranch";

    my $Git = $self->_createGit($self->_createGitRepository());

    lives_ok( sub { $Git->CreateBranch() }, "$SubTestName - Successful without parameters.");
    lives_ok( sub { $Git->CreateBranch($DummyBranch) }, "$SubTestName - Successful with branch.");
}

sub test_ExistsRemoteBranch_True {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyRemoteBranch = "issue/dummyBranch";

    my $Git = $self->_createGit($self->_createGitRepositoryForRemoteBranch($DummyRemoteBranch));
    my $WasSuccessful = $Git->ExistsRemoteBranch($DummyRemoteBranch);

    is($WasSuccessful, 1, "$SubTestName - Method returns true.");
}

sub test_ExistsRemoteBranch_False {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyRemoteBranch = "issue/dummyBranch";

    my $Git = $self->_createGit($self->_createGitRepository());
    my $WasSuccessful = $Git->ExistsRemoteBranch($DummyRemoteBranch);

    is($WasSuccessful, 0, "$SubTestName - Method returns false.");
}

sub test_ExistsLocalBranch_True {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyLocalBranch = "issue/dummyBranch";

    my $Git = $self->_createGit($self->_createGitRepositoryForLocalBranch($DummyLocalBranch));
    my $WasSuccessful = $Git->ExistsLocalBranch($DummyLocalBranch);

    is($WasSuccessful, 1, "$SubTestName - Method returns true.");
}

sub test_ExistsLocalBranch_False {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Git = $self->_createGit($self->_createGitRepositoryForLocalBranchFailure());
    my $DummyLocalBranch = "issue/dummyBranch";

    my $WasSuccessful = $Git->ExistsLocalBranch($DummyLocalBranch);

    is($WasSuccessful, 0, "$SubTestName - Method returns false.");
}

sub test_Merge_Success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $MergeBranch = "issue/MergeBranch";
    my $FeatureBranch = "issue/FeatureBranch";

    my $Git = $self->_createGit($self->_createGitRepository());

    lives_ok( sub { $Git->Merge() }, "$SubTestName - Successful without parameters.");
    lives_ok( sub { $Git->Merge($MergeBranch) }, "$SubTestName - Successful with merge branch.");
    lives_ok( sub { $Git->Merge($MergeBranch, $FeatureBranch) }, "$SubTestName - Successful with merge branch and feature branch.");
}

sub test_Push_Success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Remote = "origin";
    my $Branch = "issue/dummyBranch";

    my $Git = $self->_createGit($self->_createGitRepositoryForClone());

    lives_ok( sub { $Git->Push($Remote) }, "$SubTestName - Successful with remote.");
    lives_ok( sub { $Git->Push($Remote, $Branch) }, "$SubTestName - Successful with remote and branch.");
}

sub _createGit() {
    my $self = shift;

    my ($GitRepository) = @_;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                'DE_EPAGES::Core::API::Git',
                $aParameterList
            );

    $Mockify->addMockWithReturnValue('_createGitRepository', $GitRepository);

    return $Mockify->getMockObject();
}

sub _createGitRepository() {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                'Git::Repository',
                $aParameterList
            );

    $Mockify->addMockWithReturnValueAndParameterCheck('run', "", ['string', 'string']);

    return $Mockify->getMockObject();
}

sub _createGitRepositoryForRemoteBranch() {
    my $self = shift;

    my ($Branch) = @_;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                'Git::Repository',
                $aParameterList
            );

    $Mockify->addMockWithReturnValueAndParameterCheck('run', "upstream/$Branch\norigin/$Branch\n/$Branch\n", ['string', 'string']);

    return $Mockify->getMockObject();
}

sub _createGitRepositoryForLocalBranch() {
    my $self = shift;

    my ($Branch) = @_;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                'Git::Repository',
                $aParameterList
            );

    $Mockify->addMockWithReturnValueAndParameterCheck('run', "upstream/$Branch\norigin/$Branch\n/$Branch\n", ['string']);

    return $Mockify->getMockObject();
}

sub _createGitRepositoryForLocalBranchFailure() {
    my $self = shift;

    my ($Branch) = @_;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                'Git::Repository',
                $aParameterList
            );

    $Mockify->addMockWithReturnValueAndParameterCheck('run', "upstream/$Branch\norigin/$Branch\n/$Branch\n", ['string']);

    return $Mockify->getMockObject();
}

sub _createGitRepositoryForClone() {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                'Git::Repository',
                $aParameterList
            );

    $Mockify->addMockWithReturnValueAndParameterCheck('run', "", ['string', 'string', 'string']);

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();